# -*- encoding: utf-8 -*-

from mattermost_bot.utils import allow_only_direct_message
from mattermost_bot.utils import allowed_users
from mattermost_bot.bot import respond_to


@respond_to('^admin$')
@allow_only_direct_message()
@allowed_users('admin', 'root')
def users_access(message):
    message.reply('Access allowed!')



# -*- coding: utf-8 -*-

import os

from mattermost_bot import get_version

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.todo']
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'
project = u'mattermost_bot'
copyright = u'2016, '
version = get_version()
release = get_version()
exclude_patterns = []
pygments_style = 'sphinx'
html_theme = 'default'
htmlhelp_basename = 'mattermost_botdoc'
latex_documents = [
    ('index', 'mattermost_bot.tex', u'mattermost_bot Documentation',
     u'', 'manual'),
]
man_pages = [
    ('index', 'mattermost_bot', u'mattermost_bot Documentation',
     [u'gotlium'], 1)
]
texinfo_documents = [
    ('index', 'mattermost_bot', u'Mattermost-bot Documentation',
     u'gotlium', 'mattermost_bot', 'One line description of project.',
     'Miscellaneous'),
]



import json
import logging
import ssl
import time

import requests
import websocket

logger = logging.getLogger(__name__)


class MattermostAPI(object):
    def __init__(self, url, ssl_verify):
        self.url = url
        self.token = ""
        self.initial = None
        self.team_id = None
        self.ssl_verify = ssl_verify

    def _get_headers(self):
        return {"Authorization": "Bearer " + self.token}

    def get(self, request):
        return json.loads(
            requests.get(
                self.url + request,
                headers=self._get_headers(),
                verify=self.ssl_verify
            ).text)

    def post(self, request, data=None):
        return json.loads(requests.post(
            self.url + request,
            headers=self._get_headers(),
            data=json.dumps(data),
            verify=self.ssl_verify
        ).text)

    def login(self, name, email, password):
        props = {'name': name, 'login_id': email, 'password': password}
        p = requests.post(
            self.url + '/users/login', data=json.dumps(props),
            verify=self.ssl_verify
        )
        self.token = p.headers["Token"]
        self.load_initial_data()
        return json.loads(p.text)

    def load_initial_data(self):
        self.initial = self.get('/users/initial_load')
        self.team_id = self.initial['teams'][0]['id']

    def create_post(self, user_id, channel_id, message, files=None, pid=""):
        create_at = int(time.time() * 1000)
        return self.post(
            '/teams/%s/channels/%s/posts/create' % (self.team_id, channel_id),
            {
                'user_id': user_id,
                'channel_id': channel_id,
                'message': message,
                'create_at': create_at,
                'filenames': files or [],
                'pending_post_id': user_id + ':' + str(create_at),
                'state': "loading",
                'parent_id': pid,
                'root_id': pid,
            })

    def update_post(self, message_id, user_id, channel_id, message, files=None, pid=""):
        return self.post(
            '/teams/%s/channels/%s/posts/update' % (self.team_id, channel_id),
            {
                'id': message_id,
                'channel_id': channel_id,
                'message': message,
            })

    def channel(self, channel_id):
        return self.get('/teams/%s/channels/%s/' % (self.team_id, channel_id))

    def get_channels(self):
        return self.get('/teams/%s/channels/' % self.team_id)

    def get_profiles(self, pagination_size=100):
        profiles = {}
        start = 0
        end = start + pagination_size

        current_page = self.get('/teams/%s/users/0/%s'
                                % (self.team_id, pagination_size))
        profiles.update(current_page)
        while len(current_page.keys()) == pagination_size:
            start = end
            end += pagination_size
            current_page = self.get('/teams/%s/users/%s/%s'
                                    % (self.team_id, start, end))
            profiles.update(current_page)
        return profiles

    def me(self):
        return self.get('/users/me')

    def user(self, user_id):
        return self.get_profiles()[user_id]

    def hooks_list(self):
        return self.get('/teams/%s/hooks/incoming/list' % self.team_id)

    def hooks_create(self, **kwargs):
        return self.post(
            '/teams/%s/hooks/incoming/create' % self.team_id, kwargs)

    @staticmethod
    def in_webhook(url, channel, text, username=None, as_user=None,
                   parse=None, link_names=None, attachments=None,
                   unfurl_links=None, unfurl_media=None, icon_url=None,
                   icon_emoji=None, ssl_verify=True):
        return requests.post(
            url, data={
                'payload': json.dumps({
                    'channel': channel,
                    'text': text,
                    'username': username,
                    'as_user': as_user,
                    'parse': parse,
                    'link_names': link_names,
                    'attachments': attachments,
                    'unfurl_links': unfurl_links,
                    'unfurl_media': unfurl_media,
                    'icon_url': icon_url,
                    'icon_emoji': icon_emoji})
            }, verify=ssl_verify)


class MattermostClient(object):
    def __init__(self, url, team, email, password, ssl_verify=True, login=1):
        self.users = {}
        self.channels = {}
        self.mentions = {}
        self.api = MattermostAPI(url, ssl_verify)
        self.user = None
        self.info = None
        self.websocket = None
        self.email = None
        self.team = team
        self.email = email
        self.password = password

        if login:
            self.login(team, email, password)

    def login(self, team, email, password):
        self.email = email
        self.user = self.api.login(team, email, password)
        self.info = self.api.me()
        return self.user

    def channel_msg(self, channel, message, pid=""):
        c_id = self.channels.get(channel, {}).get("id") or channel
        return self.api.create_post(self.user["id"], c_id, message, pid=pid)

    def update_msg(self, message_id, channel, message, pid=""):
        c_id = self.channels.get(channel, {}).get("id") or channel
        return self.api.update_post(message_id, self.user["id"],
                                    c_id, message, pid=pid)

    def get_users(self):
        return self.api.get_profiles()

    def connect_websocket(self):
        host = self.api.url.replace('http', 'ws').replace('https', 'wss')
        url = host + '/users/websocket'
        self._connect_websocket(url, cookie_name='MMAUTHTOKEN')
        return self.websocket.getstatus() == 101

    def _connect_websocket(self, url, cookie_name):
        self.websocket = websocket.create_connection(
            url, header=[
                "Cookie: %s=%s" % (cookie_name, self.api.token)
            ], sslopt={
                "cert_reqs": ssl.CERT_REQUIRED if self.api.ssl_verify \
                    else ssl.CERT_NONE
            })

    def messages(self, ignore_own_msg=False, filter_action=None):
        if not self.connect_websocket():
            return
        while True:
            data = self.websocket.recv()
            if data:
                try:
                    post = json.loads(data)
                    if filter_action and post.get('event') != filter_action:
                        continue

                    if post.get('data', {}).get('post'):
                        dp = json.loads(post['data']['post'])
                        if ignore_own_msg is True and dp.get("user_id"):
                            if self.user["id"] == dp["user_id"]:
                                continue
                    yield post
                except ValueError:
                    pass

    def ping(self):
        self.websocket.ping()



import importlib
import sys
import os

DEBUG = False

PLUGINS = [
    'mattermost_bot.plugins',
]
PLUGINS_ONLY_DOC_STRING = False

BOT_URL = 'http://mm.example.com/api/v1'
BOT_LOGIN = 'bot@example.com'
BOT_PASSWORD = None
BOT_TEAM = 'devops'
SSL_VERIFY = True

IGNORE_NOTIFIES = ['@channel', '@all']
WORKERS_NUM = 10

DEFAULT_REPLY_MODULE = None
DEFAULT_REPLY = None

'''
If you use Mattermost Web API to send messages (with send_webapi()
or reply_webapi()), you can customize the bot logo by providing Icon or Emoji.
If you use Mattermost API to send messages (with send() or reply()),
the used icon comes from bot settings and Icon or Emoji has no effect.
'''
# BOT_ICON = 'http://lorempixel.com/64/64/abstract/7/'
# BOT_EMOJI = ':godmode:'


for key in os.environ:
    if key[:15] == 'MATTERMOST_BOT_':
        globals()[key[11:]] = os.environ[key]

settings_module = os.environ.get('MATTERMOST_BOT_SETTINGS_MODULE')

if settings_module is not None:
    pwd = os.getcwd()
    if pwd not in sys.path:
        sys.path.insert(0, pwd)
    settings = importlib.import_module(settings_module)
    filename = settings.__file__.replace('.pyc', '.py')
    try:
        execfile(filename)
    except NameError:
        exec(open(filename).read())

try:
    from mattermost_bot_settings import *
except ImportError:
    try:
        from local_settings import *
    except ImportError:
        pass



# -*- encoding: utf-8 -*-

from mattermost_bot.bot import respond_to, listen_to


@respond_to('^\!info$')
@listen_to('^\!info$')
def info_request(message):
    message.send('TEAM-ID: `%s`' % message.get_team_id())
    message.send('USERNAME: `%s`' % message.get_username())
    message.send('EMAIL: `%s`' % message.get_user_mail())
    message.send('USER-ID: `%s`' % message.get_user_id())
    message.send('IS-DIRECT: %s' % repr(message.is_direct_message()))
    message.send('MENTIONS: %s' % repr(message.get_mentions()))
    message.send('MESSAGE: %s' % message.get_message())



# -*- coding: utf-8 -*-

import re

from mattermost_bot.bot import listen_to
from mattermost_bot.bot import respond_to


@respond_to('hello$', re.IGNORECASE)
def hello_reply(message):
    message.reply('hello sender!')


@respond_to('hello_formatting')
@listen_to('hello_formatting$')
def hello_reply_formatting(message):
    # Format message with italic style
    message.reply('_hello_ sender!')


@listen_to('hello$')
def hello_send(message):
    message.send('hello channel!')


@listen_to('hello_decorators')
@respond_to('hello_decorators')
def hello_decorators(message):
    message.send('hello!')


@respond_to('hello_web_api', re.IGNORECASE)
def web_api_reply(message):
    attachments = [{
        'fallback': 'Fallback text',
        'author_name': 'Author',
        'author_link': 'http://www.github.com',
        'text': 'Some text here ...',
        'color': '#59afe1'
    }]
    message.reply_webapi(
        'Attachments example', attachments,
        username='Mattermost-Bot',
        icon_url='https://goo.gl/OF4DBq',
    )


@listen_to('hello_comment', re.IGNORECASE)
def hello_comment(message):
    message.comment('some comments ...')


@listen_to('hello_react', re.IGNORECASE)
def hello_react(message):
    message.react(':+1:')



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import importlib
import traceback
import logging
import json
import re

from six import iteritems

from mattermost_bot.utils import WorkerPool
from mattermost_bot import settings

logger = logging.getLogger(__name__)

MESSAGE_MATCHER = re.compile(r'^(@.*?\:?)\s(.*)', re.MULTILINE | re.DOTALL)
BOT_ICON = settings.BOT_ICON if hasattr(settings, 'BOT_ICON') else None
BOT_EMOJI = settings.BOT_EMOJI if hasattr(settings, 'BOT_EMOJI') else None


class MessageDispatcher(object):
    def __init__(self, client, plugins):
        self._client = client
        self._pool = WorkerPool(self.dispatch_msg, settings.WORKERS_NUM)
        self._plugins = plugins
        self._channel_info = {}
        self.event = None

    def start(self):
        self._pool.start()

    @staticmethod
    def get_message(msg):
        return msg.get('data', {}).get('post', {}).get('message', '').strip()

    def ignore(self, _msg):
        msg = self.get_message(_msg)
        for prefix in settings.IGNORE_NOTIFIES:
            if msg.startswith(prefix):
                return True

    def is_mentioned(self, msg):
        mentions = msg.get('data', {}).get('mentions', [])
        return self._client.user['id'] in mentions

    def is_personal(self, msg):
        channel_id = msg['data']['post']['channel_id']
        if channel_id in self._channel_info:
            channel_type = self._channel_info[channel_id]
        else:
            channel = self._client.api.channel(channel_id)
            channel_type = channel['channel']['type']
            self._channel_info[channel_id] = channel_type
        return channel_type == 'D'

    def dispatch_msg(self, msg):
        category = msg[0]
        msg = msg[1]
        text = self.get_message(msg)
        responded = False
        msg['message_type'] = '?'
        if self.is_personal(msg):
            msg['message_type'] = 'D'

        for func, args in self._plugins.get_plugins(category, text):
            if func:
                responded = True
                try:
                    func(Message(self._client, msg, self._pool), *args)
                except Exception as err:
                    logger.exception(err)
                    reply = '[%s] I have problem when handling "%s"\n' % (
                        func.__name__, text)
                    reply += '```\n%s\n```' % traceback.format_exc()
                    self._client.channel_msg(
                        msg['data']['post']['channel_id'], reply)

        if not responded and category == 'respond_to':
            if settings.DEFAULT_REPLY_MODULE is not None:
                mod = importlib.import_module(settings.DEFAULT_REPLY_MODULE)
                if hasattr(mod, 'default_reply'):
                    return getattr(mod, 'default_reply')(self, msg)
            self._default_reply(msg)

    def _on_new_message(self, msg):
        if self.ignore(msg) is True:
            return

        msg = self.filter_text(msg)
        if self.is_mentioned(msg) or self.is_personal(msg):
            self._pool.add_task(('respond_to', msg))
        else:
            self._pool.add_task(('listen_to', msg))

    def filter_text(self, msg):
        text = self.get_message(msg)
        if self.is_mentioned(msg):
            m = MESSAGE_MATCHER.match(text)
            if m:
                msg['data']['post']['message'] = m.group(2).strip()
        return msg

    def load_json(self):
        if self.event.get('data', {}).get('post'):
            self.event['data']['post'] = json.loads(
                self.event['data']['post'])
        if self.event.get('data', {}).get('mentions'):
            self.event['data']['mentions'] = json.loads(
                self.event['data']['mentions'])

    def loop(self):
        for self.event in self._client.messages(True, 'posted'):
            self.load_json()
            self._on_new_message(self.event)

    def _default_reply(self, msg):
        if settings.DEFAULT_REPLY:
            return self._client.channel_msg(
                msg['data']['post']['channel_id'], settings.DEFAULT_REPLY)

        default_reply = [
            u'Bad command "%s", You can ask me one of the '
            u'following questions:\n' % self.get_message(msg),
        ]
        docs_fmt = u'{1}' if settings.PLUGINS_ONLY_DOC_STRING else u'`{0}` {1}'

        default_reply += [
            docs_fmt.format(p.pattern, v.__doc__ or "")
            for p, v in iteritems(self._plugins.commands['respond_to'])]

        self._client.channel_msg(
            msg['data']['post']['channel_id'], '\n'.join(default_reply))


class Message(object):
    users = {}
    channels = {}

    def __init__(self, client, body, pool):
        from mattermost_bot.bot import PluginsManager

        self._plugins = PluginsManager()
        self._client = client
        self._body = body
        self._pool = pool

    def get_user_info(self, key, user_id=None):
        user_id = user_id or self._body['data']['post']['user_id']
        if not Message.users or user_id not in Message.users:
            Message.users = self._client.get_users()
        return Message.users[user_id].get(key)

    def get_username(self, user_id=None):
        return self.get_user_info('username', user_id)

    def get_user_mail(self, user_id=None):
        return self.get_user_info('email', user_id)

    def get_user_id(self, user_id=None):
        return self.get_user_info('id', user_id)

    def get_channel_name(self, channel_id=None):
        channel_id = channel_id or self._body['data']['post']['channel_id']
        if channel_id in self.channels:
            channel_name = self.channels[channel_id]
        else:
            channel = self._client.api.channel(channel_id)
            channel_name = channel['channel']['name']
            self.channels[channel_id] = channel_name
        return channel_name

    def get_team_id(self):
        return self._client.api.team_id

    def get_message(self):
        return self._body['data']['post']['message'].strip()

    def is_direct_message(self):
        return self._body['message_type'] == 'D'

    def get_busy_workers(self):
        return self._pool.get_busy_workers()

    def get_mentions(self):
        return self._body['data'].get('mentions')

    def _gen_at_message(self, text):
        return '@{}: {}'.format(self.get_username(), text)

    def _gen_reply(self, text):
        if self._body['message_type'] == '?':
            return self._gen_at_message(text)
        return text

    def _get_first_webhook(self):
        hooks = self._client.api.hooks_list()
        if not hooks:
            for channel in self._client.api.get_channels():
                if channel.get('name') == 'town-square':
                    return self._client.api.hooks_create(
                        channel_id=channel.get('id')).get('id')
        return hooks[0].get('id')

    @staticmethod
    def _get_webhook_url_by_id(hook_id):
        base = '/'.join(settings.BOT_URL.split('/')[:3])
        return '%s/hooks/%s' % (base, hook_id)

    def reply_webapi(self, text, *args, **kwargs):
        self.send_webapi(self._gen_reply(text), *args, **kwargs)

    def send_webapi(self, text, attachments=None, channel_id=None, **kwargs):
        url = self._get_webhook_url_by_id(self._get_first_webhook())
        kwargs['username'] = kwargs.get(
            'username', self.get_username(self._client.user['id']))
        kwargs['icon_url'] = kwargs.get('icon_url', BOT_ICON)
        kwargs['icon_emoji'] = kwargs.get('icon_emoji', BOT_EMOJI)
        self._client.api.in_webhook(
            url, self.get_channel_name(channel_id), text,
            attachments=attachments, ssl_verify=self._client.api.ssl_verify,
            **kwargs)

    def reply(self, text):
        self.send(self._gen_reply(text))

    def send(self, text, channel_id=None):
        return self._client.channel_msg(
            channel_id or self._body['data']['post']['channel_id'], text)

    def update(self, text, message_id, channel_id=None):
        return self._client.update_msg(
            message_id, channel_id or self._body['data']['post']['channel_id'],
            text
        )

    def react(self, emoji_name):
        self._client.channel_msg(
            self._body['data']['post']['channel_id'], emoji_name,
            pid=self._body['data']['post']['id'])

    def comment(self, message):
        self.react(message)

    def docs_reply(self, docs_format='    • `{0}` {1}'):
        reply = [docs_format.format(v.__name__, v.__doc__ or "")
                 for p, v in iteritems(self._plugins.commands['respond_to'])]
        return '\n'.join(reply)

    @property
    def channel(self):
        return self._body['data']['post']['channel_id']

    @property
    def body(self):
        return self._body



# -*- coding: utf-8 -*-

import re

from mattermost_bot.bot import respond_to


@respond_to('^busy|jobs$', re.IGNORECASE)
def busy_reply(message):
    busy = message.get_busy_workers() - 1
    message.reply('Num of busy workers is `%d`' % busy)


busy_reply.__doc__ = "Show num of busy workers"



# -*- coding: utf-8 -*-
import sys
import logging

from mattermost_bot import bot, settings


def main():
    logging.basicConfig(**{
        'format': '[%(asctime)s] %(message)s',
        'datefmt': '%m/%d/%Y %H:%M:%S',
        'level': logging.DEBUG if settings.DEBUG else logging.INFO,
        'stream': sys.stdout,
    })

    try:
        b = bot.Bot()
        b.run()
    except KeyboardInterrupt:
        pass



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import imp
import importlib
import logging
import os
import re
import time
from glob import glob

from six.moves import _thread

from mattermost_bot import settings
from mattermost_bot.dispatcher import MessageDispatcher
from mattermost_bot.mattermost import MattermostClient

logger = logging.getLogger(__name__)


class Bot(object):
    def __init__(self):
        self._client = MattermostClient(
            settings.BOT_URL, settings.BOT_TEAM,
            settings.BOT_LOGIN, settings.BOT_PASSWORD,
            settings.SSL_VERIFY
        )
        logger.info('connected to mattermost')
        self._plugins = PluginsManager()
        self._dispatcher = MessageDispatcher(self._client, self._plugins)

    def run(self):
        self._plugins.init_plugins()
        self._dispatcher.start()
        _thread.start_new_thread(self._keep_active, tuple())
        self._dispatcher.loop()

    def _keep_active(self):
        logger.info('keep active thread started')
        while True:
            time.sleep(60)
            self._client.ping()


class PluginsManager(object):
    commands = {
        'respond_to': {},
        'listen_to': {}
    }

    def __init__(self):
        pass

    def init_plugins(self):
        if hasattr(settings, 'PLUGINS'):
            plugins = settings.PLUGINS
        else:
            plugins = 'mattermost_bot.plugins'

        for plugin in plugins:
            self._load_plugins(plugin)

    @staticmethod
    def _load_plugins(plugin):
        logger.info('loading plugin "%s"', plugin)
        path_name = None
        for mod in plugin.split('.'):
            if path_name is not None:
                path_name = [path_name]
            _, path_name, _ = imp.find_module(mod, path_name)
        for py_file in glob('{}/[!_]*.py'.format(path_name)):
            module = '.'.join((plugin, os.path.split(py_file)[-1][:-3]))
            try:
                _module = importlib.import_module(module)
                if hasattr(_module, 'on_init'):
                    _module.on_init()
            except Exception as err:
                logger.exception(err)

    def get_plugins(self, category, text):
        has_matching_plugin = False
        for matcher in self.commands[category]:
            m = matcher.search(text)
            if m:
                has_matching_plugin = True
                yield self.commands[category][matcher], m.groups()

        if not has_matching_plugin:
            yield None, None


def respond_to(regexp, flags=0):
    def wrapper(func):
        PluginsManager.commands['respond_to'][re.compile(regexp, flags)] = func
        logger.info(
            'registered respond_to plugin "%s" to "%s"', func.__name__, regexp)
        return func

    return wrapper


def listen_to(regexp, flags=0):
    def wrapper(func):
        PluginsManager.commands['listen_to'][re.compile(regexp, flags)] = func
        logger.info(
            'registered listen_to plugin "%s" to "%s"', func.__name__, regexp)
        return func

    return wrapper



# -*- coding: utf-8 -*-

import time

from mattermost_bot.bot import respond_to


@respond_to('sleep (.*)')
def sleep_reply(message, sec):
    message.reply('Ok, I will be waiting %s sec' % sec)
    time.sleep(int(sec))
    message.reply('done')


sleep_reply.__doc__ = "Sleep time"



# -*- coding: utf-8 -*-

import logging

from six.moves import _thread, queue

logger = logging.getLogger(__name__)


class WorkerPool(object):
    def __init__(self, func, num_worker=10):
        self.num_worker = num_worker
        self.func = func
        self.queue = queue.Queue()
        self.busy_workers = queue.Queue()

    def start(self):
        for _ in range(self.num_worker):
            _thread.start_new_thread(self.do_work, tuple())

    def add_task(self, msg):
        self.queue.put(msg)

    def get_busy_workers(self):
        return self.busy_workers.qsize()

    def do_work(self):
        while True:
            msg = self.queue.get()
            self.busy_workers.put(1)
            self.func(msg)
            self.busy_workers.get()


def allow_only_direct_message():
    def plugin(func):
        def wrapper(message, *args, **kw):
            if not message.is_direct_message():
                return message.reply("`Only direct messages is allowed`")
            return func(message, *args, **kw)

        return wrapper

    return plugin


def allowed_users(*allowed_users_list):
    def plugin(func):
        def wrapper(message, *args, **kw):
            user = message.get_username()
            if user not in allowed_users_list:
                return message.reply("`Permission denied`")
            return func(message, *args, **kw)

        return wrapper

    return plugin



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


SECRET_KEY = 'b2!=c))9ju1q(=hlxmzb1l+2qbdjn+*cn(5(u@qn4c-0!2v+%v'
DEBUG = True

ALLOWED_HOSTS = []

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'
